@@ -1,18 +1,38 @@
+from __future__ import annotations
+
 import collections
 import datetime
 import itertools
 import typing as t
-from typing import Any, Container, Dict, Iterable, List, Optional, Tuple, Union
+from collections.abc import Mapping, MutableMapping
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Container,
+    Dict,
+    Generator,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import isodate
 
 import rdflib.plugins.sparql
-from rdflib.compat import Mapping, MutableMapping
 from rdflib.graph import ConjunctiveGraph, Graph
 from rdflib.namespace import NamespaceManager
 from rdflib.plugins.sparql.parserutils import CompValue
 from rdflib.term import BNode, Identifier, Literal, Node, URIRef, Variable
 
+if TYPE_CHECKING:
+    from rdflib.paths import Path
+
+
+_AnyT = TypeVar("_AnyT")
+
 
 class SPARQLError(Exception):
     def __init__(self, msg: Optional[str] = None):
@@ -80,8 +100,8 @@ class Bindings(MutableMapping):
             d = d.outer
         return i
 
-    def __iter__(self):
-        d = self
+    def __iter__(self) -> Generator[str, None, None]:
+        d: Optional[Bindings] = self
         while d is not None:
             yield from d._d
             d = d.outer
@@ -162,7 +182,6 @@ class FrozenBindings(FrozenDict):
         self.ctx = ctx
 
     def __getitem__(self, key: Union[Identifier, str]) -> Identifier:
-
         if not isinstance(key, Node):
             key = Variable(key)
 
@@ -197,7 +216,7 @@ class FrozenBindings(FrozenDict):
 
     def forget(
         self, before: "QueryContext", _except: Optional[Container[Variable]] = None
-    ):
+    ) -> FrozenBindings:
         """
         return a frozen dict only of bindings made in self
         since before
@@ -220,14 +239,14 @@ class FrozenBindings(FrozenDict):
             ),
         )
 
-    def remember(self, these):
+    def remember(self, these) -> FrozenBindings:
         """
         return a frozen dict only of bindings in these
         """
         return FrozenBindings(self.ctx, (x for x in self.items() if x[0] in these))
 
 
-class QueryContext(object):
+class QueryContext:
     """
     Query context - passed along when evaluating the query
     """
@@ -236,7 +255,7 @@ class QueryContext(object):
         self,
         graph: Optional[Graph] = None,
         bindings: Optional[Union[Bindings, FrozenBindings, List[Any]]] = None,
-        initBindings: Optional[Dict[Variable, Identifier]] = None,
+        initBindings: Optional[Mapping[str, Identifier]] = None,
     ):
         self.initBindings = initBindings
         self.bindings = Bindings(d=bindings or [])
@@ -292,7 +311,18 @@ class QueryContext(object):
             )
         return self._dataset
 
-    def load(self, source: URIRef, default: bool = False, **kwargs):
+    def load(self, source: URIRef, default: bool = False, **kwargs: Any) -> None:
+        """
+        Load data from the source into the query context's.
+
+        :param source: The source to load from.
+        :param default: If `True`, triples from the source will be added to the
+            default graph, otherwise it will be loaded into a graph with
+            ``source`` URI as its name.
+        :param kwargs: Keyword arguments to pass to
+            :meth:`rdflib.graph.Graph.parse`.
+        """
+
         def _load(graph, source):
             try:
                 return graph.parse(source, format="turtle", **kwargs)
@@ -320,13 +350,12 @@ class QueryContext(object):
                 # Unsupported left operand type for + ("None")
                 self.graph += self.dataset.get_context(source)  # type: ignore[operator]
         else:
-
             if default:
                 _load(self.graph, source)
             else:
-                _load(self.dataset, source)
+                _load(self.dataset.get_context(source), source)
 
-    def __getitem__(self, key) -> Any:
+    def __getitem__(self, key: Union[str, Path]) -> Optional[Union[str, Path]]:
         # in SPARQL BNodes are just labels
         if not isinstance(key, (BNode, Variable)):
             return key
@@ -335,7 +364,7 @@ class QueryContext(object):
         except KeyError:
             return None
 
-    def get(self, key: Variable, default: Optional[Any] = None):
+    def get(self, key: str, default: Optional[Any] = None) -> Any:
         try:
             return self[key]
         except KeyError:
@@ -352,7 +381,7 @@ class QueryContext(object):
         else:
             return FrozenBindings(self, self.bindings.items())
 
-    def __setitem__(self, key: Identifier, value: Identifier) -> None:
+    def __setitem__(self, key: str, value: str) -> None:
         if key in self.bindings and self.bindings[key] != value:
             raise AlreadyBound()
 
@@ -384,7 +413,7 @@ class Prologue:
     A class for holding prefixing bindings and base URI information
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.base: Optional[str] = None
         self.namespace_manager = NamespaceManager(Graph())  # ns man needs a store
 
